Extension { #name : 'EUCString' }

{ #category : 'Formatting' }
EUCString class >> charSize [

"Returns the number of bytes that make up a character"
^1

]

{ #category : 'Comparing' }
EUCString >> _EUCcompare: aCharCollection [

"Returns -1 if self < aCharCollection, returns 0 if self = aCharCollection,
 and returns 1 if self > aCharCollection."

<primitive: 78>

^ self _primitiveFailed: #_EUCcompare: args: { aCharCollection }

]

{ #category : 'Deprecated' }
EUCString >> _isValidPathTermName [

"Returns true if the receiver is a valid term in a path expression."

self deprecated:'_isValidPathTermName deprecated (EUCString not supported path terms)'.
^ false

]

{ #category : 'Comparing' }
EUCString >> < aCharCollection [

"Returns true if the receiver collates before the argument.  Returns false
 otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) == -1]
  ifFalse: [ ^super < aCharCollection ]

]

{ #category : 'Comparing' }
EUCString >> <= aCharCollection [

"Returns true if the receiver collates before the argument or if all of the
 corresponding Characters in the receiver and argument are equal.  Returns
 false otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) <= 0]
  ifFalse: [ ^super <= aCharCollection ]

]

{ #category : 'Comparing' }
EUCString >> = aCharCollection [

"Returns true if all of the corresponding Characters in the receiver and
 argument are equal.  Returns false otherwise."

<primitive: 77>

^ super = aCharCollection

]

{ #category : 'Comparing' }
EUCString >> > aCharCollection [

"Returns true if the receiver collates after the argument.  Returns false
 otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) == 1]
  ifFalse: [ ^super > aCharCollection ]

]

{ #category : 'Comparing' }
EUCString >> >= aCharCollection [

"Returns true if the receiver collates after the argument or if all of the
 corresponding Characters in the receiver and arguments are equal.  Returns
 false otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) >= 0]
  ifFalse: [ ^super >= aCharCollection ]

]

{ #category : 'Adding' }
EUCString >> add: aCharOrCharCollection [

"Appends aCharOrCharCollection to the receiver.  The argument
 aCharOrCharCollection must be a CharacterCollection or a kind of
 AbstractCharacter."

<primitive: 242>

^ super add: aCharOrCharCollection

]

{ #category : 'Adding' }
EUCString >> addAll: aCharOrCharCollection [

"Equivalent to add: aCharOrCharCollection."

<primitive: 242>

^ super addAll: aCharOrCharCollection

]

{ #category : 'Adding' }
EUCString >> addLast: aCharOrCharCollection [

"Equivalent to add: aCharOrCharCollection."

<primitive: 242>

^ super addLast: aCharOrCharCollection

]

{ #category : 'Formatting' }
EUCString >> asEUCString [

"Returns the receiver."

"This reimplementation of from JapaneseString is crucial !."

^self

]

{ #category : 'Converting' }
EUCString >> asSymbol [

"Returns a Symbol that contains the same bytes as the receiver."

^ (DoubleByteString withAll: self) asSymbol

]

{ #category : 'Converting' }
EUCString >> asSymbolKind [

"Returns a copy of the receiver as an instance of Symbol."

^ self asSymbol

]

{ #category : 'Accessing' }
EUCString >> at: anIndex [

"Returns the JISCharacter at the specified index."

<primitive: 244>

(anIndex _isSmallInteger)
ifTrue:[ ^ self _errorIndexOutOfRange: anIndex]
ifFalse:[^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError

]

{ #category : 'Updating' }
EUCString >> at: anIndex put: aCharacter [

"Stores aCharacter at the specified location."

<primitive: 243>
| charClass |
charClass := aCharacter class .
(charClass == JISCharacter or:[ charClass == Character]) ifFalse:[
  ^ self at: anIndex put: aCharacter asJISCharacter .
  ].
(anIndex _isSmallInteger)
ifTrue:[ ^ self _errorIndexOutOfRange: anIndex]
ifFalse:[^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #at:put: args: { anIndex . aCharacter } .
self _uncontinuableError

]

{ #category : 'Formatting' }
EUCString >> charSize [

"Returns the number of bytes that make up a character"
^1

]

{ #category : 'Accessing' }
EUCString >> codePointAt: anIndex [

^ (self at: anIndex) eucValue

]

{ #category : 'Updating' }
EUCString >> codePointAt: anIndex put: aValue [

^ self at: anIndex put:(JISCharacter withEUCValue: aValue)

]

{ #category : 'Searching' }
EUCString >> indexOf: aCharacter startingAt: startIndex [

"Returns the index of the first occurrence of aCharacter in the receiver,
 not preceding startIndex.  If the receiver does not contain aCharacter,
 this returns zero."

<primitive: 79>

^super indexOf: aCharacter startingAt: startIndex

]

{ #category : 'Adding' }
EUCString >> insertAll: aCharOrCharCollection at: anIndex [

"Inserts aCharOrCharCollection at the specified index."

<primitive: 241>

anIndex _validateClass: SmallInteger.
(anIndex > (self size + 1))
ifTrue:[ ^ self _errorIndexOutOfRange: anIndex]
ifFalse:[ ^ super insertAll: aCharOrCharCollection at: anIndex]

]

{ #category : 'Formatting' }
EUCString >> printOn: aStream [

"Puts a displayable representation of the receiver on the given stream."

1 to: (self size min: 200) do:[:j |
  aStream nextPut: (Character codePoint: (self codePointAt: j))
].

]

{ #category : 'Formatting' }
EUCString >> printString [

"Returns an EUCString whose contents are a displayable representation of the
 receiver."

| ws str |

str := Unicode16 new.
ws := PrintStream printingOn: str.
self printOn: ws.
^str

]

{ #category : 'Accessing' }
EUCString >> size [

"Returns the number of JISCharacters in the receiver."

<primitive: 76>

self _primitiveFailed: #size .
self _uncontinuableError

]

{ #category : 'Updating' }
EUCString >> size: anInteger [

"Changes the size of the receiver to anInteger.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to nil."

<primitive: 245>

(anInteger _isSmallInteger)
ifTrue:[^ self _errorIndexOutOfRange: anInteger]
ifFalse:[^ self _errorNonIntegerIndex: anInteger].

self _primitiveFailed: #size: args: { anInteger } .
self _uncontinuableError

]
